(require "draw-lib.ss")

(define-signature hangmanS (hangman-repl g-hangman-repl))

(define hangmanU
  (unit/sig
    hangmanS
    (import drawS plt:userspace^)

    (define (check-proc proc f n arg# arg-err)
      (unless (procedure? f)
	(error proc "procedure expected as ~s arg; given ~s" arg# f))
      (unless (and (number? (arity f)) (= (arity f) n))
	(error proc
	  "procedure of ~a expected as ~s arg; given procedure of ~s args" 
	  arg-err arg# (arity f))))

    (define (check proc str reveal draw-next-part f)
      (unless (and (list? str) (andmap symbol? str))
	(error proc "word (list-of-a..z) expected as 1st arg; given ~s" str))
      (check-proc proc reveal         3 '2nd "3 arguments")
      (f proc)
      (check-proc proc draw-next-part 1 '4th "1 argument"))

    (define (hangman-repl str reveal make-word draw-next-part)
      ;; -- check args
      (check 'hangman-repl str reveal draw-next-part
	(lambda (p)
	  (check-proc p make-word 5 '3rd "5 arguments")))
      ;; ---
      (stop)
      (start 200 400)
      (set! pieces-left NUMBER-OF-GUESSES)
      (draw-next-part (vector-ref PARTS pieces-left))
      (let repl ((known (make-word '_ '_ '_ '_ '_)) (guesses null))
	(let* ((guess (read)) (new (reveal str known guess)))
	  (cond
	    ((member guess guesses) (printf "You're losing your memory."))
	    ((equal? known new) 
	     (set! pieces-left (- pieces-left 1))
	     (draw-next-part (vector-ref PARTS pieces-left)))
	    (else
	      (printf "Good. Here is what we know: ~a~n" new)))
	  (cond
	    ((= pieces-left 0) (printf "This is the end my only friend.~n"))
	    ((equal? new str) (printf "You won. Good bye."))
	    (else (repl new (cons guess guesses)))))))

    (define (g-hangman-repl str reveal copy draw-next-part)
      (check 'hangman-repl str reveal draw-next-part
	(lambda (p)
	  (check-proc p copy 1 '3rd "1 argument")))
      (stop)
      (start 200 400)
      (set! pieces-left NUMBER-OF-GUESSES)
      (draw-next-part (vector-ref PARTS pieces-left))
      (let repl ((known (copy str)) (guesses null))
	(printf "Take another guess! ")
	(let* ((guess (read))
	       (l (cons guess guesses))
	       (new (reveal str known guess)))
	  (cond
	    ((member guess guesses) (printf "You're losing your memory."))
	    ((equal? known new) 
	     (set! pieces-left (- pieces-left 1))
	     (draw-next-part (vector-ref PARTS pieces-left)))
	    (else (printf "Good. Here is what we know: ~a~n" new)))
	  (printf "~s~n" pieces-left)
	  (cond
	    ((= pieces-left 0) (printf "This is the end my only friend.~n"))
	    ((equal? new str) (printf "You won. Good bye."))
	    (else (repl new l))))))

    (define PARTS
      (vector 'right-leg 'left-leg 'left-arm 'right-arm 'body 'head 'noose))

    (define pieces-left 0)
    (define NUMBER-OF-GUESSES 6)))

(compound-unit/sig (import (PLT : plt:userspace^))
  (link
    [DRAW  : drawS  ((require-unit/sig "draw-lib.ss") PLT)]
    [HANG  : hangmanS (hangmanU DRAW PLT)])
  (export (open DRAW) (open HANG)))
