(require "draw-sig.ss")
(require "pingp-sig.ss")

(define pingpU
  (unit/sig pingpS
    (import drawS plt:userspace^)

;; Exported Functions
;; ------------------
    
    (define (tracer f pos speed color e)
      ; ---
      (check-proc 'tracer f 3 '1st "3 arguments")
      (unless (and (posn? pos) (rgb? color) (number? e))
	(error 'tracer
	  (format "~a ~a given: ~s ~s ~s ~s ~s"
	    "expects arguments of type "
	    "<procedure>, <posn>, <vec>, <color>, <num>;"
	    f pos speed color e)))
      ; ---
      (start2 EAST SOUTH)
      (let dl ((x0 (f pos speed 0)) (s 1))
	; (printf "(dl ~s ~s)~n" x0 s)
	(if (> s e)
	    #t
	    (let ((x1 (f pos speed s)))
	      (and
		(draw-solid-disk x1 3 color)
		(draw-solid-line x0 x1)
		(dl x1 (+ s 1)))))))

    (define (trace-ball ball ball-posn f e)
      ; ---
      (check-proc 'tracer-ball ball-posn 1 '2nd "1 argument")
      (check-proc 'tracer-ball f         2 '3rd "2 arguments")
      (unless (integer? e)	
	(error 'trace-ball "expected integer as 4th argument, given: ~s" e))
      ; ---
      (start2 EAST SOUTH)
      (let dl ((ball0 ball) (s 1))
	(if (> s e)
	    #t
	    (let ((ball1 (f ball0 1)))
	      (and
		(draw-solid-disk (ball-posn ball1) 3 RED)
		(draw-solid-line (ball-posn ball0) (ball-posn ball1))
		(dl ball1 (+ s 1)))))))

(define (trace-bouncing-ball make-ball ball-posn ball-speed move)
  (unset-pad)
  (set-trace)
  (check 'trace-bouncing-ball make-ball ball-posn ball-speed move)
  (play2 make-ball ball-posn ball-speed move))

(define (play make-ball ball-posn ball-speed move)
  (set-pad)
  (unset-trace)
  (check 'play make-ball ball-posn ball-speed move)
  (play2 make-ball ball-posn ball-speed move))

;; Testing Whether the Ball will Hit the Paddle:
;; ---------------------------------------------

(define (landed-on-paddle? ballp)
  (error 'landed-on-paddle?
    "eval (play make-ball ball-posn ball-speed move) first"))

;; Setting Up the Game
;; -------------------
(define (change-speed speed)
  (if (and (integer? speed) (> speed 0))
      (set! SLEEP (/ 10 speed))
      (error 'change-speed "expects positive integer; given ~s" speed)))

(define (change-wind switch)
  (if (and (integer? switch) (> switch 0))
      (set! SWITCH switch)
      (error 'change-wind "expects positive integer; given ~s" switch)))

;; Auxiliary Definitions
;; =====================

;; Global Properties (initialized by set-up!)
;; ------------------------------------------

(define BALL-COLOR RED)
(define PAD-COLOR BLUE)
(define (set-pad) (set! PAD-COLOR BLUE)) 
(define (unset-pad) (set! PAD-COLOR WHITE)) 
(define TRACE-COLOR WHITE)
(define (set-trace) (set! TRACE-COLOR GREEN)) 
(define (unset-trace) (set! TRACE-COLOR WHITE)) 

(define SLEEP .15)
(define SWITCH 10000)

(define NORTH 0)
(define SOUTH 500)
(define WEST 0)
(define EAST 500)
(define CENTER (quotient EAST 2))
(define PADDLE-X EAST)
(define PADDLE-Y (quotient SOUTH 2))

;; the following can't be abstracted because of (set! <id> ...)
(define (change-width n)
  (unless (and (integer? n) (>= n 200))
    (error 'change-width "expects an integer >= 200; rec'd: ~s" n))
  (set! EAST n))

(define (change-height n)
  (unless (and (integer? n) (>= n 200))
    (error 'change-height "expects an integer >= 200; rec'd: ~s" n))
  (set! SOUTH n))

;; Checking and Playing
;; --------------------

(define (check-proc proc f n arg# arg-err)
  (unless (procedure? f)
    (error proc "procedure expected as ~s arg; given ~s" arg# f))
  (unless (and (number? (arity f)) (= (arity f) n))
    (error proc
      "procedure of ~a expected as ~s arg; given procedure of ~s args" 
      arg-err arg# (arity f))))

(define (check s make-ball ball-posn ball-speed move)
  (check-proc s make-ball  2 '1st "2 arguments")
  (check-proc s ball-posn  1 '2nd "1 argument")
  (check-proc s ball-speed 1 '3rd "1 argument")
  (check-proc s move       2 '4th "2 arguments"))

(define (play2 make-ball ball-posn ball-speed move)
  (ready-to-go?)
  (let* ((rn-10-10 (lambda ()
		     (let ((n (random 20)))
		       (if (< n 10) (- n 10) (- n  9)))))
	 (make-rn-speed (lambda () (make-posn (rn-10-10) (rn-10-10))))
	 (ball (make-ball (make-posn (quotient EAST 2) (quotient SOUTH 2))
		 (make-rn-speed)))
	 (PAD_Y0 (- (quotient SOUTH 2) (quotient PAD_LENGTH 2)))
	 (west (box (make-posn PAD_DIST_WALL                        PAD_Y0)))
	 (east (box (make-posn (- EAST (+ PAD_WIDTH PAD_DIST_WALL)) PAD_Y0)))
	 (start-time (current-seconds)))
    (set! landed-on-paddle? (make-landed-on-paddle east west))
    ;; the loop
    (let play ((ball0 ball) (p0 (ball-posn ball)) (i 1))
      (draw-paddle (unbox west))
      (draw-paddle (unbox east))
      (draw-ball p0)
      (sleep SLEEP)
      (if (or (< (posn-x p0) 0) (< EAST (posn-x p0)))
	  (begin (printf "You kept the ball in play for ~s seconds." 
		   (- (current-seconds) start-time))
	    (stop))
	  (let* ((ball1 (move ball0 1))
		 (p1 (ball-posn ball1))
		 ;; paddle move?
		 (mc (ready-mouse-click @VP))
		 (new-posn (and mc (mouse-click-posn mc))))
	    (clear-ball p0)
	    (clear-paddle (unbox west))
	    (clear-paddle (unbox east))
	    (draw-solid-line p0 p1 TRACE-COLOR)
	    (cond
	      ((not new-posn) (void))
	      ((in-west-zone? new-posn) 
	       (set-box! west (new-paddle (unbox west) (posn-y new-posn))))
	      ((in-east-zone? new-posn)
	       (set-box! east (new-paddle (unbox east) (posn-y new-posn)))))
	    (if (zero? (modulo i SWITCH))
		(begin
		  (draw-solid-disk p0 BALL-RADIUS GREEN)
		  (play (make-ball p1 (make-rn-speed)) p1 1))
		(play ball1 p1 (add1 i))))))))

(define (protect objs move-objs remove-objs-hit-paddle remove-outside-objs objs-posn)
  (set! PAD_DIST_WALL (quotient EAST 10))
  (set! PADDLE-X (- EAST PAD_DIST_WALL PAD_WIDTH))
  (let* ((objs# (length objs))
	 (PAD_Y0 (- (quotient SOUTH 2) (quotient PAD_LENGTH 2)))
	 (east (make-posn (- EAST (+ PAD_WIDTH PAD_DIST_WALL)) PAD_Y0)))
    (set! landed-on-paddle?
      (lambda (p)
	(and (= (posn-x p) PADDLE-X)
	     (<= (posn-y east) (posn-y p) (+ (posn-y east) PAD_LENGTH)))))
    (ready-to-go?)
    ;; the loop
    (let PLAY ((ball0 objs) (p0 (objs-posn objs)) (hits 0))
      (draw-paddle east)
      (for-each draw-ball p0)
      (sleep SLEEP)
      (if (null? ball0)
	  (begin (printf "You caught ~s balls, but ~s hit the wall."
		         hits (- objs# hits))
		 (stop))
	  (let* ((ball1 (move-objs ball0))
		 (ball2 (remove-objs-hit-paddle ball1))
		 (ball3 (remove-outside-objs ball2))
		 (p1    (objs-posn ball3))
		 ;; paddle move?
		 (mc    (ready-mouse-click @VP)))
	    (for-each clear-ball p0)
	    (clear-paddle east)
	    (when mc
	      (let ((new-posn (mouse-click-posn mc)))
		(when (in-east-zone? new-posn)
		  (set! east (new-paddle east (posn-y new-posn))))))
	    (PLAY ball3 p1 (+ hits (- (length ball1) (length ball2)))))))))

(define (ready-to-go?)
  (start2 EAST SOUTH)
  (draw-solid-rect   (make-posn (- (quotient EAST 2) 100) 10) 200 20 GREEN)
  ((draw-string @VP) (make-posn (- (quotient EAST 2)  65) 20)
   "Click anywhere when ready!" RED)
  (let loop () (unless (ready-mouse-click @VP) (loop)))
  (draw-solid-rect   (make-posn (- (quotient EAST 2) 100) 10) 200 20 WHITE))
 
(define (start2 x y)
  (start x y)
  (draw-solid-rect (make-posn 0 0) EAST SOUTH GREEN))

;; A Ball Representation
;; ---------------------
(define BALL-RADIUS 5)
;(define (draw-ball p)
;  (set! draw-ball ((draw-pixmap-posn "Gifs/redball.gif") @VP))
;  (draw-ball p))
;(define (clear-ball p)
;  (set! clear-ball ((clear-pixmap-posn "Gifs/redball.gif") @VP))
;  (clear-ball p))
(define (draw-ball p) (draw-solid-disk p BALL-RADIUS RED))
(define (clear-ball p) (draw-solid-disk p BALL-RADIUS WHITE))

;; A Paddle Representation
;; -----------------------
(define PAD_WIDTH  3)
(define PAD_LENGTH 50)
(define PAD_DIST_WALL  3)

(define (draw-paddle paddle)
  (draw-solid-rect paddle PAD_WIDTH PAD_LENGTH PAD-COLOR))
(define (clear-paddle paddle)
  (draw-solid-rect paddle PAD_WIDTH PAD_LENGTH WHITE))

(define (new-paddle paddle new-y)
  (make-posn (posn-x paddle) new-y))

(define (make-landed-on-paddle east west)
  (lambda (pball)
    (let ((y-pad (posn-y
		   (unbox
		     (cond
		       ((= EAST (posn-x pball)) east)
		       ((= WEST (posn-x pball)) west)
		       (else (error 'landed-on-paddle
			       "this ball has not reached a wall: ~s" pball)))))))
      (<= y-pad (posn-y pball) (+ y-pad PAD_LENGTH)))))
      
(define (in-west-zone? p) (<= 0 (posn-x p) CENTER))
(define (in-east-zone? p) (<= CENTER (posn-x p) EAST))

))

(compound-unit/sig (import (PLT : plt:userspace^))
  (link
    [DRAW  : drawS  ((require-unit/sig "draw-lib.ss") PLT)]
    [PINGP : pingpS (pingpU DRAW PLT)])
  (export (open DRAW) (open PINGP)))
